//-*-C++-*- Header file for the pSpecial Class
// Copyright (C) 1997 by John P. Weiss
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
// This file is unreadable, because the comment-docifier is
// brainedead.  Try a "make srcdocs" to generate the
// HTML-documentation.
//
#ifndef _pSpecial_h_
#define _pSpecial_h_ 1
#include <qwidget.h>
#include <qsize.h>
#include <qlabel.h>
#include <qpopmenu.h>
#include "PixButton.h"
#include "paneldefs.h"




/**
  {\bf pSpecial} contains a set of special control buttons.  Two of
  these are for volume control.  The others are designed for
  configuration and such.

  Two of the buttons are actually private members of this class.  They
  are connected to two popup menus which select the time display mode
  and paly mode.  The menus emit their state via the two signals of
  this class.

  @memo Panel of special buttons.
  @see PixButton, PixData
  */
class pSpecial : public QWidget
{
	 //
	 Q_OBJECT

   public:
	 ///
	 PixButton *vol_dnb;
	 /**
	   \Ref{PixButton}s for use in volume control.
	   @memo
	   */
	 PixButton *vol_upb;
	 ///
	 QPushButton *configb;
	 ///
	 QPushButton *playlstb;
	 /**
	   Buttons to be connected to dialog boxes.  \Ref{configb},
	   \Ref{playlstb}, and \Ref{progb} are intended for
	   configuration, catalog editing, and player programming,
	   respectively.
	   @memo
	   */
	 QPushButton *progb;

	 /**
	   Class Constructor
	   @param parent The parent object of this widget.
	   @param compact Flag to indicate label style.  {\em
	   compact=true} tells {\bf pSpecial} to use short, abbreviated
	   button labels.
	   @memo
	   */
	 pSpecial( QWidget *parent, bool compact=false );

	 /**
	   Class Destructor
	   @memo
	   */
	 ~pSpecial( void );

	 /**
	   Moves the object, performing some internal bookkeeping at the
	   same time.
	   @memo
	   */
	 inline void moveAll( int x, int y );

	 /**
	   Overloaded version.
	   @memo
	   */
	 inline void moveAll( QPoint pos );

	 /**
	   @return The point in the x-direction where this widget ends.
	   @memo
	   */
	 inline QPoint xEndPt(void);

	 /**
	   @return The point in the y-direction where this widget ends.
	   @memo
	   */
	 inline QPoint yEndPt(void);

	 /**
	   Changes the palette of this object to {\em p}.
	   @memo
	   */
	 void changePalette( QPalette& p );

	 /**
	   Changes the font used on the button labels.
	   @memo
	   */
	 void changeFont( QFont& f );
	 
   public slots:

	 /**
	   {\em Public Slot}

	   Slot to open the time mode menu.  Provided so that other
	   objects can connect to it.	   
	   @memo [{\tt public slot}]
	   */
	 void tmenuOpen();

	 /**
	   {\em Public Slot}

	   Overloaded version.  Contains the position at which to open
	   the menu.
	   @memo [{\tt public slot}]
	   */
	 void tmenuOpen(QPoint);

	 /**
	   {\em Public Slot}

	   Slot to open the play mode menu.  Provided so that other
	   objects can connect to it.	   
	   @memo [{\tt public slot}]
	   */
	 void pmenuOpen();

	 /**
	   {\em Public Slot}

	   Overloaded version.  Contains the position at which to open
	   the menu.
	   @memo [{\tt public slot}]
	   */
	 void pmenuOpen(QPoint);

   signals:

	 /**
	   {\em Signal}

	   Signal to indicate a change in the time display mode.  Emits a
	   \Ref{TimeMode} {\bf enum}.  Used by the time mode menu.
	   @memo [{\tt signal}]
	   */
	 void newTimeMode(int);

	 /**
	   {\em Signal}

	   Signal to indicate a change in the play mode.  Emits a
	   \Ref{PlayMode} {\bf enum}.  Used by the play mode menu.
	   @memo [{\tt signal}]
	   */
	 void newPlayMode(int);

	 /**
	   {\em Signal}

	   Emitted when a mouse press-event occurs anywhere else on the
	   panel outside of one of the widgets.  Contains the current
	   mouse position, in global screen coordinates.
	   @memo [{\tt signal}] */
	 void pressed(QPoint);

	 /**
	   {\em Signal}

	   Emitted when a mouse move-event occurs anywhere else on the
	   panel outside of one of the widgets.  Contains the current
	   mouse position, in global screen coordinates.
   	   @memo [{\tt signal}] */
	 void moving(QPoint);


	 //
   private:
	 void createChildren( bool compact );
	 void locateChildren(void);
	 void moveChildren(void);
	 void createMenus(void);
	 void doRescale(QFont& f);
	 void mousePressEvent( QMouseEvent *me );
	 void mouseMoveEvent( QMouseEvent *me );
	 //
	 QPopupMenu tmenu, pmenu;
	 QPushButton *tmodeb, *pmodeb;
	 QLabel *voltxt;
	 QPoint ftbw_p, vu_p, vd_p, vtxt_p,
	   prg_p, cfg_p, plst_p, tm_p, pm_p;
	 QSize ftbw_s;
	 int tmea_id,tme_id,tmra_id,tmr_id,pmn_id,pmrpa_id,pmrpo_id,pms_id;
	 double hscale,wscale;
	 // Only static const members can be initialized.  BUT - this
	 // member will be *identical* for all objects declared under
	 // this class because of the "static"!
	 static const int bsep=WIDGET_SEP;

   private slots:
      // Used internally to generate the mode signals.
	 void tm_ea();
	 void tm_e();
	 void tm_ra();
	 void tm_r();
	 void pm_n();
	 void pm_rpa();
	 void pm_rpo();
	 void pm_s();
};




//----------------------------------------------------------
//
// Inlined methods.
//
//----------------------------------------------------------




// Moves the entire Feature widget, including the children.
//
inline void pSpecial::moveAll( int x, int y )
{
  ftbw_p.setX( x );
  ftbw_p.setY( y );
  move( ftbw_p );
}


// Overloaded version of the other function.
//
inline void pSpecial::moveAll( QPoint pos )
{
  ftbw_p = pos;
  move( ftbw_p );
}


// Returns a point specifying where to put the next widget
//
inline QPoint pSpecial::xEndPt( void )
{
  return( QPoint(ftbw_s.width(),0) + ftbw_p );
}


// Returns a point specifying where to put the next widget, in the
// y-direction.
//
inline QPoint pSpecial::yEndPt( void )
{
  return( QPoint(0,ftbw_s.height()) + ftbw_p );
}




#endif //_pSpecial_h_
//||||||||||||||||||||||||||||||||
//
// End
// 
